#include "WolframAlpha.hpp"

#include "Utility.hpp"
#include <cpr/cpr.h>
#include <ctre.hpp>
#include <nlohmann/json.hpp>
#include <string>

WolframAlpha::WolframAlpha(const std::string& query, double amount) : m_json(fetchJson(query)), m_amount(amount) {}

double WolframAlpha::getCalories() {
  if (auto match = ctre::search<R"((?<=total calories\s)\d+)">(getPod())) {
    return std::stod(match.data()) * m_amount;
  }
  throw std::runtime_error("Calorie information not found.");
}

double WolframAlpha::getProtein() {
  if (auto match = ctre::search<R"((?<=protein\s)\d+\s*\w+)">(getPod())) {
    const std::string val  = std::string(match.to_view().substr(0, match.begin() - match.end()));
    std::string       unit = std::string(ctre::search<R"(\w+$)">(match.to_view()));

    return unit == "g" ? std::stod(val) * m_amount : std::stod(val) / 1000 * m_amount;
  }
  throw std::runtime_error("Protein information not found.");
}

double WolframAlpha::getCarbs() {
  if (auto match = ctre::search<R"((?<=total carbohydrates\s)\d+\s*\w+)">(getPod())) {
    const std::string val  = std::string(match.to_view().substr(0, match.begin() - match.end()));
    std::string       unit = std::string(ctre::search<R"(\w+$)">(match.to_view()));

    return unit == "g" ? std::stod(val) * m_amount : std::stod(val) / 1000 * m_amount;
  }
  throw std::runtime_error("Carbohydrates information not found.");
}

double WolframAlpha::getFat() {
  if (auto match = ctre::search<R"((?<=total fat\s)\d+\s*\w+)">(getPod())) {
    const std::string val  = std::string(match.to_view().substr(0, match.begin() - match.end()));
    std::string       unit = std::string(ctre::search<R"(\w+$)">(match.to_view()));

    return unit == "g" ? std::stod(val) * m_amount : std::stod(val) / 1000 * m_amount;
  }
  throw std::runtime_error("Fat information not found.");
}

nlohmann::json WolframAlpha::fetchJson(const std::string& query) {
  const auto    WOLFRAM_KEY = utl::getEnv("NUTRI_WA_KEY");
  cpr::Response json        = Get(cpr::Url{"https://api.wolframalpha.com/v2/query"},
                                  cpr::Parameters{{"input", query + "100g nutrition"}, {"appid", WOLFRAM_KEY}, {"output", "JSON"}, {"format", "plaintext"}});
  return nlohmann::json::parse(json.text);
}

std::string WolframAlpha::getPod() {
  return to_string(m_json["queryresult"]["pods"][1]["subpods"][0]["plaintext"]);
}

nlohmann::json m_json{};
double         m_amount{};
